#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>

#include "BuildLog.h"
#ifndef NetLoadD_Defaults
#define NetLoadD_Defaults
 #define NetLoadD_Version_Major 1
 #define NetLoadD_Version_Minor 5
 #define NetLoadD_Version_Release 1
 #define NetLoadD_Version_Build GOD_Build
#endif

char NLD_Version[64];

void NetLoadD_Version()
 {
 sprintf(NLD_Version,"%d.%d.%d/%d (%s)", NetLoadD_Version_Major,
	NetLoadD_Version_Minor, NetLoadD_Version_Release,
	NetLoadD_Version_Build, "Linux");
 }

void VersionInfo(int nl=0)
 {
 NetLoadD_Version();
 printf("netloadd %s Developmental Build",NLD_Version);
 if (nl>0) printf("\n");
 }

#include "netloadd.h"
TNetLoadD_Config *Config;
#include "netloadd_analysis.h"
void CatchSignal(int sig);
void KillNetload(int);

TInterface *interface;

int main (int argnum, char **args)
 {
 NetLoadD_Version();
 Config=new TNetLoadD_Config();
 if (argnum<2)
  {
  printf("Usage %s [options] interface\n",args[0]);
  return 0;
  }
 else if (argnum>2)
  {
  if (Config->AutoConfig(args,1,argnum-2)==0)
   {
   printf("Error: incorrect/misunderstood command line parameters\n");
   return 0;
   }
  }
 else if (argnum==2) 
  {
  if (Config->AutoConfig(args,1,1)!=0)
   return 0;
  }
 if (Config->Commentary>0)
  {
  VersionInfo(0);
  printf(" Starting\n");
  }
 if (Config->Debug>0) Config->PrintConfig();
 if (Config->Syslog>0)
  {
  char sln[64];
  sprintf(sln,"netloadd %s\0",args[argnum-1]);
  SL_Open(sln);
  }
 interface=new TInterface(args[argnum-1],10);
 if (interface->Sample()==0)
  {
  printf("Error: device %s not found in /proc/net/dev\n",args[argnum-1]);
  SL_Close();
  return 0;
  }
 (void) signal( SIGINT, CatchSignal);
 (void) signal( SIGKILL, CatchSignal);
 while (1)
  {
  sleep(Config->Delay);
  if (interface->Sample()==0)
   {
   printf("Error fetching %s info from /proc/dev/net\n",
	interface->Device);
   if (Config->QuitOnError>0) KillNetload(0);
   }
  if (Config->Debug>0)
   {
  //interface->PrintListStat();
  char one[1024],two[1024];
  sprintf(one,"%s: Rx %lu bytes, Tx %lu bytes in %lu seconds\0",
	interface->Device,interface->RxDiff,interface->TxDiff,
	interface->TimeDiff);
  sprintf(two,"%s: Total Rx %lu, Total Tx %lu\0",
	interface->Device,interface->LastReceive,interface->LastTransmit);
  if (Config->Syslog>0)
   {
   SL_Write(one);
   SL_Write(two);
   }
  if (Config->Commentary>0)
   {
   printf("%s\n%s\n",one,two);
   }
   }
  if (Config->FileWrite>0)
   {
   char filename[128];
   for (int z=0; z<128; z++) filename[z]=0;
   strcpy(filename,"/tmp/netloadd.");
   strcat(filename,interface->Device);
   interface->WriteToFile(filename,1);
   }
  if (Config->MySQL>0)
   {
   int GoMySQL=1;
   if (Config->SQL_DataLoaded==0)
    {
    printf("Error: SQL Data Not Loaded\n");
    if (Config->QuitOnError>0) return 0;
    else GoMySQL=0;
    }
   #ifdef NetLoadD_Module_MySQL
   MYSQL *s;
   if (GoMySQL>0)
   s=SQL_Connect(Config->SQL_Server,Config->SQL_Username,Config->SQL_Password,
	Config->SQL_Database,Config->SQL_Hostname,Config->SQL_Hash);
   if (!s) 
    {
    printf("Error opening MySQL connection\n");
    if (Config->QuitOnError>0) return 0;
    else GoMySQL=0;
    }
   if (GoMySQL>0)
    {
    SQL_Post(interface->LastSampled,interface->RxDiff,interface->TxDiff,
	interface->LastReceive, interface->Device);
    SQL_Disconnect(s);
    }
   #else
   printf("Error: MySQL Not Compiled into NetLoadD\n");
   #endif
   }
  }
 return 1;
 }

void CatchSignal(int sig)
 {
 printf("Caught Signal - Exixing\n");
 if (Config->Syslog>0)
  {
  SL_Write("Caught Signal - Exiting");
  SL_Close();
  }
 KillNetload(0);
 }

void KillNetload(int flag=1)
 {
 delete interface;
 exit(flag);
 }
